#	include	"Simulator.h"

StandardDRAMDevice	::	StandardDRAMDevice ()
	{
	AccessLatency = TransferRate = 0;
	UpStreamComponent = NULL;
	Transacting = 0;
	}

StandardDRAMDevice	::	~StandardDRAMDevice ()
	{
	}

bool
StandardDRAMDevice	::	PrintConfiguration (
	int		Format,
	FILE	*fp)
	{
	PrintValue(Format, fp, "AccessLatency", AccessLatency);
	PrintValue(Format, fp, "TransferRate", TransferRate);
	return ( true );
	}

bool
StandardDRAMDevice	::	PrintStatistics (
	int		Format,
	FILE	*fp)
	{
	PrintValue(Format, fp, "Accesses", Accesses);
	PrintValue(Format, fp, "BytesRead", BytesRead);
	PrintValue(Format, fp, "BytesWritten", BytesWritten);
	PrintValue(Format, fp, "FrontSideBusBusyCycles", BusBusyCycles);
	PrintValue(Format, fp, "FrontSideBusIdleCycles", BusIdleCycles);
	PrintPercent(Format, fp, "FrontSideBusUtilization", BusBusyCycles, BusIdleCycles);
	PrintValue(Format, fp, "DRAMBusyCycles", DRAMBusyCycles);
	PrintValue(Format, fp, "DRAMIdleCycles", DRAMIdleCycles);
	PrintPercent(Format, fp, "DRAMUtilization", DRAMBusyCycles, DRAMIdleCycles);
	return ( true );
	}

bool
StandardDRAMDevice	::	Initialize(
	uint32	aAccessLatency,
	uint32	aTransferRate)
	{
	AccessLatency = aAccessLatency;
	TransferRate = aTransferRate;

	return ( true );
	}

bool
StandardDRAMDevice	::	Connect(
	Component	*c)
	{
	UpStreamComponent = c;
	return ( true );
	}

int
StandardDRAMDevice	::	SimulateOneClockCycle()
	{
	if(FrontSideBus.Busy())
		++BusBusyCycles;
	else
		++BusIdleCycles;

	if(DRAM.Busy())
		++DRAMBusyCycles;
	else
		++DRAMIdleCycles;

	return(1);
	}

bool
StandardDRAMDevice	::	ScheduleOperation(
	bool		IsWrite,
	SimAddress	sa,
	int			Bytes,
	int			StartAfter,
	int			*UntilStart,
	int			*UntilFinish)
	{
	int			Temp1, Temp2;
	Transacting = StartAfter;
	(*UntilStart) = Transacting;
	// Schedule bus stuff
	if(IsWrite)
		{
		FrontSideBus.RequestNewSchedule(Transacting,
			2*TransferRate + TransferRate*(Bytes>>4),
			0, &Temp1, &Temp2);
		Transacting = Temp2;
		}	
	else
		{
		FrontSideBus.RequestNewSchedule(Transacting,
			2*TransferRate,
			0, &Temp1, &Temp2);
		Transacting = Temp2;
		}

	//DRAM.RequestNewSchedule	(Transacting, AccessLatency, 0, &Temp1, &Temp2);
	Transacting += AccessLatency;

	// if read, post back data.
	if(!IsWrite)
		{
		Transacting = Temp2;
		FrontSideBus.RequestNewSchedule(
			Transacting, TransferRate*(Bytes>>4),
			0, &Temp1, &Temp2);
		Transacting = Temp2;
		}

	(*UntilFinish) = Transacting;

	++Accesses;
	if(IsWrite)
		BytesWritten += Bytes;
	else
		BytesRead += Bytes;
	return(true);
	}

