
// OSF Binary form (Ecoff)
typedef	struct	_AlphaBinaryFileHeader
	{
	le_uint16		MagicNumber;
	le_uint16		NumberOfSections;
	le_uint32		TimeData;
	le_uint64		SymbolPointer;
	le_uint32		NumberOfSymbols;
	le_uint16		OptionalHeader;
	le_uint16		Flags;
	}	AlphaBinaryFileHeader;

typedef	struct	_AlphaBinaryAOutHeader
	{
	le_uint16		MagicNumber;
	le_uint16		Version;
	le_uint16		BuildRevision;
	le_uint16		Padding;
	le_uint64		TextSize;
	le_uint64		DataSize;
	le_uint64		BssSize;
	le_uint64		InitialPC;
	le_uint64		TextStart;
	le_uint64		DataStart;
	le_uint64		BssStart;
	le_uint32		GPRMask;
	le_uint32		FPRMask;
	le_uint64		GPValue;
	}	AlphaBinaryAOutHeader;

typedef	struct	_AlphaBinarySectionHeader
	{
	char		Name[8];
	le_uint64		PhysicalAddress;
	le_uint64		VirtualAddress;
	le_uint64		Size;
	le_uint64		SectionPointer;
	le_uint64		RelativePointer;
	le_uint64		Options;
	le_uint16		NumberOfRelocatableSymbols;
	le_uint16		NumberOfOptions;
	le_uint32		Flags;
	}	AlphaBinarySectionHeader;

typedef	struct	_AlphaBinaryExecFile
	{
	AlphaBinaryFileHeader		FileHeader;
	AlphaBinaryAOutHeader		AOutHeader;
	AlphaBinarySectionHeader	SectionHeaders[20];
	}	AlphaBinaryExecFile;


// Linux binary form (ELF)
typedef	struct	_LinuxAlphaBinaryFileHeader
	{
	byte		MagicNumber[16];
	le_uint64		Padding;
	le_uint64		InitialPC;
	le_uint64		ProgramHeaderOffset;
	le_uint64		SectionHeaderOffset;
	le_uint16		FileType;
	le_uint16		Machine;
	le_uint16		HeaderSize;
	le_uint16		ProgramHeaderTableSize;
	le_uint16		ProgramHeaderTableCount;
	le_uint16		SectionHeaderTableSize;
	le_uint16		NumberOfSections;
	le_uint16		SectionHeaderStringIndex;
	}	LinuxAlphaBinaryFileHeader;

typedef	struct	_LinuxAlphaBinaryProgramHeader
	{
	le_uint32		Type;
	le_uint32		Flags;
	le_uint64		SectionPointer;
	le_uint64		VirtualAddress;
	le_uint64		PhysicalAddress;
	le_uint64		Size;
	le_uint64		SizeInMemory;
	le_uint64		Alignment;
	}	LinuxAlphaBinaryProgramHeader;

typedef	struct	LinuxAlphaBinarySectionHeader
	{
	le_uint32		Name;
	le_uint32		Type;
	le_uint64		Flags;
	le_uint64		VirtualAddress;
	le_uint32		Size;
	le_uint32		EntrySize;
	le_uint32		Link;
	le_uint32		Info;
	le_uint32		SectionPointer;
	le_uint32		Alignment;
	}	LinuxAlphaSectionHeader;

typedef	struct	_LinuxAlphaBinaryExecFile
	{
	LinuxAlphaBinaryFileHeader		FileHeader;
	LinuxAlphaBinaryProgramHeader	ProgramHeaders[20];
	}	LinuxAlphaBinaryExecFile;

class	AlphaProcessor;

// File loader.  Use the main "LoadFromDevice" calls.  These will load
// both ECOFF (OSF) and ELF (Linux) Alpha binaries.

class	Loader
	{
	public:
		static	bool	DumpHeader(AlphaBinaryFileHeader	*fh);
		static	bool	DumpHeader(AlphaBinaryAOutHeader	*ah);
		static	bool	DumpHeader(AlphaBinarySectionHeader	*sh);
		static	bool	DumpHeader(AlphaBinaryExecFile		*ef);

		static	bool	DumpHeader(LinuxAlphaBinaryFileHeader	*fh);
		static	bool	DumpHeader(LinuxAlphaBinaryProgramHeader	*ph);
		static	bool	DumpHeader(LinuxAlphaBinarySectionHeader	*sh);
		static	bool	DumpHeader(LinuxAlphaBinaryExecFile		*ef);

		static	bool	LoadSegmentIntoMemorySystem(int	fd,
					AlphaBinaryExecFile			*ef,
					AlphaBinarySectionHeader	*Section,
					AlphaProcessor				*p);
		static	bool	LoadELFSegmentIntoMemorySystem(int	fd,
					LinuxAlphaBinaryExecFile			*ef,
					LinuxAlphaBinaryProgramHeader		*Section,
					AlphaProcessor						*p);
		static	int			FormulateInitialStack (
				uint64				StackBase,
				int					argc,
				char				*argv[],
				char				*argp[],
				unsigned			char	*Output);
		static	bool	LoadELFFromDevice(int	fd,
			AlphaProcessor		*p,
			char				**argv,
			char				**argp);
		static	bool	LoadFromDevice(int	fd,
			AlphaProcessor		*p,
			char		**argv = NULL,
			char		**argp = NULL);
		static	bool	LoadFromDevice(char	*Filename,
			AlphaProcessor		*p,
			char		**argv = NULL,
			char		**argp = NULL);
	};


