#ifdef	DEBUG_MEMORY

#	include	"Standard.h"

SafeMemory	sfa;

extern	"C"
	{
#	include	<stddef.h>
#	include	<stdlib.h>
	}

typedef	struct	_MemAction
	{
	int		AllocOrDelete;
	int		LineNumber;
	char	*FileName;
	int		MemSize;
	void	*vptr;
	struct	_MemAction	*Related;
	struct	_MemAction	*Prev, *Next;
	}	MemAction;

static	MemAction		*MemorySegments;
static	bool	WatchMemory = true;
static	char	TempFileName[1024];
static	int		TempLineNumber;

MemAction	*CreateMemAction (
	char	*FileName,
	int		LineNumber,
	int		Action,
	int		Size,
	void	*vptr)
	{
	MemAction	*MA;

	MA = (MemAction*)malloc(sizeof(MemAction));
	printf("CreateMemAction(%s, %d, %d, %d, %x): %x\n",
		FileName, LineNumber, Action, Size, (uint)vptr, (uint)MA);
	memset((void*)MA, 0, sizeof(MemAction));
	if(FileName[0])
		{
		MA->FileName = (char*)malloc(strlen(FileName)+4);
		strcpy(MA->FileName, FileName);
		}
	MA->AllocOrDelete = Action;
	MA->vptr = vptr;
	MA->MemSize = Size;
	MA->LineNumber = LineNumber;
	return ( MA );
	}

bool
FreeMemAction (
	MemAction	*MA )
	{
	/* very destructive */
	//printf("Feeing MA: %x\n", (uint) MA);
	if ( ! MA )
		return ( false );
	//FreeMemAction(MA->Prev);
	//FreeMemAction(MA->Next);
	//FreeMemAction(MA->Related);
	if ( MA->FileName )
		free (MA->FileName);
	free( MA );
	return ( true );
	}

bool
InitMemoryWatch()
	{

	MemorySegments = NULL;

	return ( true );
	}

static
bool
__s_isascii(	char	ch	)
	{
	if ( (ch>='A')&&(ch<='Z'))
		return ( true );
	if ( (ch>='a')&&(ch<='z'))
		return ( true );
	if ( (ch>='0')&&(ch<='9'))
		return ( true );
	return ( false );
	}

bool
PrintUnfreed(void	*data, uint	Size)
	{
	uint32	*p32;
	uint16	*p16;
	uint	Index;
	char	*pc;

	switch ( Size )
		{
		case	2:	p16 = (uint16*) data;
					fprintf(stderr, "%4.4x / %d ",*p16,*p16);
					break;
		case	4:	p32 = (uint32*) data;
					fprintf(stderr, "%8.8x / %d ",*p32,*p32);
					break;
		}
	Index = 0;
	pc = (char*)data;
	while ( Index < Size )
		{
		if ( Index > 50 )
			break;
		if(__s_isascii(*pc))
			fprintf(stderr, "%2.2x(%c),", *pc, *pc);
		else
			fprintf(stderr, "%2.2x(.),", *pc);
		++pc;
		++Index;
		}
	fprintf(stderr, "\n");
	return ( true );
	}

bool
CloseMemoryWatch()
	{
	uint	Index;
	uint	Count;
	MemAction	*MA;

	Index = 0; Count = 0;

	MA = MemorySegments;

	while ( MA )
		{
		fprintf (stderr, "%s/%d Memory : %8.8x : %d - Not Freed\n",
				MA->FileName,
				MA->LineNumber,
				(uint)MA->vptr,
				MA->MemSize);
		fflush ( stderr );
		PrintUnfreed(MA->vptr, MA->MemSize);
		FreeMemAction(MA);
		MA = MA->Next;
		++Count;
		}
	FreeMemAction(MemorySegments);	
	MemorySegments = NULL;
	return ( Count );	
	}

bool
RegisterMemory (size_t	Size, void	*vptr)
	{
	MemAction	*MA;

	MA = CreateMemAction(TempFileName, TempLineNumber, 1, Size, vptr);

	MA->Next = MemorySegments;
	if (MemorySegments)
		MemorySegments->Prev = MA;
	MemorySegments = MA;
	return ( true );
	}

bool
UnRegisterMemory ( void *vptr )
	{
	MemAction	*MA;

	MA = MemorySegments;

	while ( MA )
		{
		if ( MA->vptr == vptr )
			{
			if ( MemorySegments == MA )
				MemorySegments = MA->Next;
			if ( MA->Prev )
				MA->Prev->Next = MA->Next;
			if ( MA->Next )
				MA->Next->Prev = MA->Prev; 
			MA->Next = MA->Prev = MA->Related = NULL;
			FreeMemAction(MA);
			return ( true );
			}
		MA = MA->Next;
		}
	return ( false );
	}
#undef	new
#undef	delete

void	* operator	new(size_t	x)	throw()
	{
	void	*vptr;

	if ( ! x )
		{
		fprintf(stdout, "Attempt to allocate zero length block in %s/%d\n",
			TempFileName, TempLineNumber);
		exit(1);
		}

	vptr = malloc(x);
	if(!vptr)
		{
		printf("unable to allocate %d bytes / %d k / %d meg\n",
			x, x/1024, x/(1024*1024));
		}
	if(WatchMemory)
	RegisterMemory (x, vptr);
	return ( vptr );

	}

void	 operator	delete	( void	*vptr ) throw()
	{
	if ( ! vptr )
		{
		fprintf(stdout, "Attempt to delete NULL pointer %s/%d\n",
			TempFileName, TempLineNumber);
		return;
		}
	if(WatchMemory)
	if (!UnRegisterMemory(vptr))
		{
		if(MemorySegments)
			printf("Possible Duplicate Delete in File: %s Line %d vptr=%x\n",
			TempFileName, TempLineNumber, (uint)vptr);
		}
	free(vptr);
	}

int	snew(
	char	*file,
	int		line)
	{
	if(!WatchMemory)	return(0);
	strcpy(TempFileName, file);
	TempLineNumber = line;
	return(0);	// make the else part go..
	}

int	 sdelete	(
	char	*file,
	int		line )
	{
	if(!WatchMemory)	return(0);
	strcpy(TempFileName, file);
	TempLineNumber = line;
	return(0);	// make the else part go..
	}


void
EnableMemoryWatch()
	{
	WatchMemory = true;
	}

void
DisableMemoryWatch()
	{
	WatchMemory = false;
	}
#endif

