#	include	"Simulator.h"

// Various configuraitons parameters

uint	NumberOfProcessors = 1, StackSize = 0x800000;
uint	L1IBanks = 4, L1ILineSize = 64, L1IBankSize = 64;
bool	L1IWriteBack = false;
uint	L1IFSBSC = 0, L1IFSBBS = 1;
uint	L1DBanks = 4, L1DLineSize = 64, L1DBankSize = 64;
bool	L1DWriteBack = true;
uint	L1DFSBSC = 0, L1DFSBBS = 1;
uint	L2CBanks = 4, L2CLineSize = 128, L2CBankSize = 2048;
bool	L2CWriteBack = true;
uint	L2CFSBSC = 0, L2CFSBBS = 1;
uint	SNOOPYFSBSC = 4, SNOOPYFSBBS = 2;
uint	DRAMSC = 1000, DRAMBS = 10, DRAMPageSize = 4096;
int		OutputFormat = HumanFormat;
char	OutputFile[1024] = "stdout";
uint	L1IAccess = 0, L1DAccess = 0, L2Access = 12;
bool	Quiet = false;
Counter	StopSimulationAfter = 0;

CommandLineArgument	Args[]	=
	{
		{	"Quiet",		claBool,	&Quiet	},
		{	"Processors",		clauInteger,	&NumberOfProcessors },
		{	"StackSize",		clauInteger,	&StackSize },
		{	"L1-I-Banks",		clauInteger,	&L1IBanks },
		{	"L1-I-LineSize",	clauInteger,	&L1ILineSize },
		{	"L1-I-BankSize",	clauInteger,	&L1IBankSize },
		{	"L1-I-WriteBack",	claBool,		&L1IWriteBack },
		{	"L1-I-FSB-SC",		clauInteger,	&L1IFSBSC },
		{	"L1-I-FSB-BS",		clauInteger,	&L1IFSBBS },
		{	"L1-I-Access",		clauInteger,	&L1IAccess },
		{	"L1-D-Banks",		clauInteger,	&L1DBanks },
		{	"L1-D-LineSize",	clauInteger,	&L1DLineSize },
		{	"L1-D-BankSize",	clauInteger,	&L1DBankSize },
		{	"L1-D-WriteBack",	claBool,		&L1DWriteBack },
		{	"L1-D-FSB-SC",		clauInteger,	&L1DFSBSC },
		{	"L1-D-FSB-BS",		clauInteger,	&L1DFSBBS },
		{	"L1-D-Access",		clauInteger,	&L1DAccess },
		{	"L2-C-Banks",		clauInteger,	&L2CBanks },
		{	"L2-C-LineSize",	clauInteger,	&L2CLineSize },
		{	"L2-C-BankSize",	clauInteger,	&L2CBankSize },
		{	"L2-C-WriteBack",	claBool,		&L2CWriteBack },
		{	"L2-C-FSB-SC",		clauInteger,	&L2CFSBSC },
		{	"L2-C-FSB-BS",		clauInteger,	&L2CFSBBS },
		{	"L2-Access",		clauInteger,	&L2Access },
		{	"SNOOPY-FSB-SC",	clauInteger,	&SNOOPYFSBSC },
		{	"SNOOPY-FSB-BS",	clauInteger,	&SNOOPYFSBSC },
		{	"DRAM-SC",			clauInteger,	&DRAMSC },
		{	"DRAM-BS",			clauInteger,	&DRAMBS },
		{	"DRAM-PageSize",	clauInteger,	&DRAMPageSize },
		{	"OutputFormat",		claInteger,		&OutputFormat },
		{	"OutputFile",		claString,		OutputFile },
		{	"StopSimulationAfter",	claCounter,	&StopSimulationAfter	},
		{	NULL,				clauInteger, NULL }
	};

Simulator	*theSimulator;	// must be a global and must be named theSimulator

int
main (
	int		argc,
	char	*argv[],
	char	*argp[])
	{
	// usage: sim3 program [args..]
	int	Index, Processors, vargc;
	uint64	ProcessorStackSize;
	char	s[1024];

	if(argc<2)
		{
		printf("usage: %s [sim-args] program [args..]\n", argv[0]);
		printf("       use -h to list available sim-args\n");
		return(1);
		}
	else
	if((vargc = ParseCommandLine(Args, argc, argv))<0)
		{
		printf("usage: %s [sim-args] program [args..]\n", argv[0]);
		printf("       use -h to list available sim-args\n");
		return(1);
		}
	if((vargc)>=argc)
		{
		printf("usage: %s [sim-args] program [args..]\n", argv[0]);
		printf("       use -h to list available sim-args\n");
		return(1);
		}	

	Processors = NumberOfProcessors;
	ProcessorStackSize = StackSize;

	// Initialize system-board devices
	theSimulator = new Simulator;
	theSimulator->StopSimulationAfter = StopSimulationAfter;
	StandardDRAMDevice			*DRAM = new StandardDRAMDevice;
	DisembodiedSnoopyControlUnit	*SCU = new DisembodiedSnoopyControlUnit;
	DisembodiedCache			*L1Is, *L2s;
	DisembodiedSnoopyCache		 *L1Ds;
	SingleIssueProcessor		*Proc;
	
	L1Is = new DisembodiedCache[Processors];
	L1Ds = new DisembodiedSnoopyCache[Processors];
	L2s = new DisembodiedCache;
	Proc = new SingleIssueProcessor[Processors];

	sprintf(s, "Processor-L2");
	L2s->SetName(s);
	L2s->Initialize(
							L2CLineSize, L2CBankSize, L2CBanks,
							L2CWriteBack,
							L2CFSBSC, L2CFSBBS, L2Access);
	for(Index=0;Index<Processors;Index++)
		{
		if ( (Index+1) < Processors )
			Proc[Index].NextProcessor = &Proc[Index+1];
		else
			Proc[Index].NextProcessor = NULL;
		Proc[Index].FirstProcessor = &Proc[0];

		sprintf(s, "Processor-%d-L1-I", Index);
		L1Is[Index].SetName(s);
		L1Is[Index].Initialize(
							L1ILineSize, L1IBankSize, L1IBanks,
							L1IWriteBack,
							L1IFSBSC, L1IFSBBS, L1IAccess);

		sprintf(s, "Processor-%d-L1-D", Index);
		L1Ds[Index].SetName(s);
		L1Ds[Index].Initialize(
							L1DLineSize, L1DBankSize, L1DBanks,
							L1DWriteBack,
							L1DFSBSC, L1DFSBBS, L1DAccess);


		sprintf(s, "Processor-%d", Index);
		Proc[Index].SetName(s);

		L1Is[Index].ConnectToUpStreamComponent(&Proc[Index]);
		L1Is[Index].ConnectToDownStreamComponent(L2s);
		Proc[Index].ConnectInstructionMemory(&L1Is[Index]);

		L1Ds[Index].ConnectToUpStreamComponent(&Proc[Index]);
		L1Ds[Index].ConnectToDownStreamComponent(SCU);
		Proc[Index].ConnectDataMemory(&L1Ds[Index]);

		SCU->AddConnectedCache(&L1Ds[Index]);
		}

	L2s->ConnectToUpStreamComponent(SCU);
	L2s->ConnectToDownStreamComponent(DRAM);
	
	SCU->Initialize( L1DLineSize, SNOOPYFSBSC, SNOOPYFSBBS);
	SCU->SetName("SnoopyControlUnit");

	DRAM->Initialize(DRAMSC, DRAMBS);
 	DRAM->SetName("DRAM");
 
	SCU->ConnectMemorySystem(L2s);
	DRAM->Connect(L2s);

	theSimulator->SetName("Machine");

	// The final thing to set is the log-file output
	FILE	*fp = stdout;
	if(strcmp(OutputFile, "stdout"))
		{
		if(!strcmp(OutputFile, "stderr"))
			fp = stderr;
		else
			{
			fp = fopen(OutputFile, "a+");
			if(!fp)
				{
				printf("error creating logfile: %s\n", OutputFile);
				return(1);
				}
			fseek(fp, 0, SEEK_END);
			}
		}	
	// Print the system board devices configuration	

	if ( ! Quiet )
		{
		theSimulator->PrintConfiguration(OutputFormat, fp);
		SCU->PrintConfiguration(OutputFormat, fp);
		for(Index=0;Index<Processors;Index++)
			{
			Proc[Index].PrintConfiguration(OutputFormat, fp);
			L1Is[Index].PrintConfiguration(OutputFormat, fp);
			L1Ds[Index].PrintConfiguration(OutputFormat, fp);
			}
		L2s->PrintConfiguration(OutputFormat, fp);
		DRAM->PrintConfiguration(OutputFormat, fp);
		}
	Proc[0].theAlphaProcessor = new AlphaProcessor;

	printf("Executable: %s\n", argv[vargc]);
	if(!Proc[0].theAlphaProcessor->Initialize(argv[vargc], &argv[vargc], NULL))
		{
		printf("usage: sim4mp [args] prog ...\n");
		exit(1);
		}

	++Processor::NumberOfActiveProcessors;

	theSimulator->RegisterComponent(theSimulator);
	theSimulator->RegisterComponent(DRAM);
	theSimulator->RegisterComponent(SCU);
	for(Index=0;Index<Processors;Index++)
		{
		theSimulator->RegisterComponent(&Proc[Index]);
		theSimulator->RegisterComponent(&L1Is[Index]);
		theSimulator->RegisterComponent(&L1Ds[Index]);
		theSimulator->RegisterComponent(L2s);
		}

	theSimulator->LiveProcessors = 1;
	theSimulator->StartThread = NULL;
	theSimulator->NumberOfCPUs = Processors;
	theSimulator->FastForward = true;

	theSimulator->Simulate();


	if ( ! Quiet )
		{
		printf("\n");
		theSimulator->PrintStatistics(OutputFormat, fp);
		SCU->PrintStatistics(OutputFormat, fp);

		for(Index=0;Index<Processors;Index++)
			{
			Proc[Index].PrintStatistics(OutputFormat, fp);
			L1Is[Index].PrintStatistics(OutputFormat, fp);
			L1Ds[Index].PrintStatistics(OutputFormat, fp);
			L2s->PrintStatistics(OutputFormat, fp);
			}
		DRAM->PrintStatistics(OutputFormat, fp);

		fprintf(fp, "\n");
		}

	exit(0);
	return(0);
	}


